//#include <windows.h> jun
//#pragma hdrstop jun

#include "f00Cube.h"

#include <vector>
#include <sys/stat.h>
#include <fstream.h>

#include "f00TargaLoader.h"
#include "f00TextureObject.h"

//jun start

#include <Alert.h>
#include <Application.h>

static const char dataname[6][10] =	
{
	"tex01.tga",
	"tex02.tga",
	"tex03.tga",
	"tex04.tga",
	"tex05.tga",
	"tex06.tga"
};

extern char path[B_PATH_NAME_LENGTH];

//jun end

// We will store the textures in here.
std::vector< f00TextureObject* > textureList;


//
// Convience functions....
//

f00TextureObject*
loadTargaTexture( const char* pFileName )
{
	assert( pFileName != 0 );

	struct stat statBuffer;
	if( stat( pFileName, &statBuffer ) != 0 )
		assert( 0 );

	int bufferSize( statBuffer.st_size );
	assert( bufferSize > 0 );

	char* pFileBuffer = new char[ bufferSize ];

	ifstream is( pFileName, ios::in | ios::binary );
	assert( is );
	is.read( pFileBuffer, bufferSize );

	f00TargaLoader tgaLoader;
	GLubyte* pImageRGBA = 0;
	int width;
	int height;
	if( tgaLoader.loadImage( pFileBuffer, bufferSize ) )
		pImageRGBA = tgaLoader.extractRGBA( width, height );
	assert( pImageRGBA != 0 );

	delete[] pFileBuffer;

	return new f00TextureObject( new f00RAMTexture( pImageRGBA, width, height ) );
}


void
insertSorted( std::vector< int >& indexList, std::vector< GLfloat >& zValList, int index, GLfloat zVal )
{
	// Just walk the list linearly.  We only have a few to add so this is good enough....
	int loop;
	for( loop = 0; loop < (int)indexList.size(); loop++ )
		if( zVal > zValList[ loop ] )
			break;
	indexList.insert( indexList.begin() + loop, index );
	zValList.insert( zValList.begin() + loop, zVal );
}




f00Cube::f00Cube(	GLfloat x,
						GLfloat y,
						GLfloat z,
						GLfloat yaw,
						GLfloat pitch,
						GLfloat roll	)
:	f00Object( x, y, z, yaw, pitch, roll ),
	d_hiQualityFlag( false )
{
	size( 5.0 );
	setTexturedMode();

	// Load up some textures.
	f00TextureObject* pWorkTexture;

//jun begin
	int i;
	char *dest;
	char datadir[B_PATH_NAME_LENGTH] = "";
	char datapath[B_PATH_NAME_LENGTH] = "";

	dest = strrchr(path,'/');
	strncpy(datadir, path, dest - path + 1);

	for (i = 0; i < 6; i++)
	{
		strcpy(datapath,datadir);
		strcat(datapath , dataname[i]);
		pWorkTexture = loadTargaTexture( datapath );

		if (pWorkTexture == 0)
		{
			(new BAlert("","Missing...datafile","OK"))->Go();
			exit(0);
		}
		else
		{
			textureList.insert( textureList.end(), pWorkTexture );
		}
	}

//jun end

	setTextureQualityFlag( true, true );
}


f00Cube::~f00Cube()
{
	for( int loop = (int)textureList.size() - 1; loop > 1; loop-- )//jun
		delete( textureList[ loop ] );
	textureList.erase( textureList.begin(), textureList.end() );
}


void
f00Cube::renderObject()
{
	// Smack the base object.
	f00Object::renderObject();

	// Just In Case (tm).
	assert( textureList.size() == 6 );

	if( isTexturedMode() )
		glEnable( GL_TEXTURE_2D );
	else
		glDisable( GL_TEXTURE_2D );

	//
	// Order the faces back to front (use feedback to get the Z for now).
	//

	#define DEF_CubeFeedBackSize  		6 * ( 3 + 1 + 1 + 1 )
	GLfloat zValsForFaces[ DEF_CubeFeedBackSize ];
	glFeedbackBuffer( DEF_CubeFeedBackSize, GL_3D, zValsForFaces );
	glRenderMode( GL_FEEDBACK );

	// Slap in some points (face centers).
	GLfloat halfSize( size() / 2 );
	glPassThrough( 0.0 );	glBegin( GL_POINTS );	glVertex3f(       0.0,       0.0,  halfSize );	glEnd();
	glPassThrough( 1.0 );	glBegin( GL_POINTS );	glVertex3f( -halfSize,       0.0,       0.0 );	glEnd();
	glPassThrough( 2.0 );	glBegin( GL_POINTS );	glVertex3f(  halfSize,       0.0,       0.0 );	glEnd();
	glPassThrough( 3.0 );	glBegin( GL_POINTS );	glVertex3f(       0.0,       0.0, -halfSize );	glEnd();
	glPassThrough( 4.0 );	glBegin( GL_POINTS );	glVertex3f(       0.0, -halfSize,       0.0 );	glEnd();
	glPassThrough( 5.0 );	glBegin( GL_POINTS );	glVertex3f(       0.0,  halfSize,       0.0 );	glEnd();

	int hits( glRenderMode( GL_RENDER ) );
	if( hits != DEF_CubeFeedBackSize )
	{
		// Something is not visible, just abort.
		return;
	}

	// Some spot checks, JIC....
	assert( zValsForFaces[ 0 ] == GL_PASS_THROUGH_TOKEN );
	assert( zValsForFaces[ 6 ] == GL_PASS_THROUGH_TOKEN );

	// Sort by Z value.
	std::vector< int > indexList;
	std::vector< GLfloat > zValList;
	for( int loop = 0; loop < 6; loop++ )
		insertSorted( indexList, zValList, (int)zValsForFaces[ 1 + loop * 6 ], zValsForFaces[ 5 + loop * 6 ] );

	for( int loop = 0; loop < 6; loop++ )
	{
		switch( indexList[ loop ] )
		{
			case 0:
				// Front
				textureList[ 0 ] -> select();
				drawPanel();
				break;

			case 1:
				// Left
				textureList[ 1 ] -> select();
				glPushMatrix();
					glRotatef( -90.0, 0.0, 1.0, 0.0 );
					drawPanel();
				glPopMatrix();
				break;

			case 2:
				// Right
				textureList[ 2 ] -> select();
				glPushMatrix();
					glRotatef( 90.0, 0.0, 1.0, 0.0 );
					drawPanel();
				glPopMatrix();
				break;

			case 3:
				// Back
				textureList[ 3 ] -> select();
				glPushMatrix();
					glRotatef( 180.0, 0.0, 1.0, 0.0 );
					drawPanel();
				glPopMatrix();
				break;

			case 4:
				// Bottom
				textureList[ 4 ] -> select();
				glPushMatrix();
					glRotatef( 90.0, 1.0, 0.0, 0.0 );
					drawPanel();
				glPopMatrix();
				break;

			case 5:
				// Top
				textureList[ 5 ] -> select();
				glPushMatrix();
					glRotatef( -90.0, 1.0, 0.0, 0.0 );
					drawPanel();
				glPopMatrix();
				break;

			default:
				assert( 0 );
				break;
		}
	}

}


void
f00Cube::drawPanel()
{
	GLfloat halfSize( size() / 2 );
	bool isSpray( isSprayColorMode() );

	if( !isSpray )
		glColor4f( 1.0, 1.0, 1.0, 1.0 );

	glBegin( GL_QUADS );

		if( isSpray )
			glColor4f( 1.0, 0.0, 0.0, 0.5 );
		else
			glTexCoord2f( 0.0, 0.0 );
		glVertex3f( -halfSize, -halfSize,  halfSize );

		if( isSpray )
			glColor4f( 0.0, 1.0, 0.0, 0.5 );
		else
			glTexCoord2f( 1.0, 0.0 );
		glVertex3f(  halfSize, -halfSize,  halfSize );

		if( isSpray )
			glColor4f( 0.0, 0.0, 1.0, 0.5 );
		else
			glTexCoord2f( 1.0, 1.0 );
		glVertex3f(  halfSize,  halfSize,  halfSize );

		if( isSpray )
			glColor4f( 1.0, 1.0, 1.0, 0.5 );
		else
			glTexCoord2f( 0.0, 1.0 );
		glVertex3f( -halfSize,  halfSize,  halfSize );

	glEnd();
}


void
f00Cube::setTextureQualityFlag( bool hiQualityFlag, bool forceUpdate )
{
	if( !forceUpdate && d_hiQualityFlag == hiQualityFlag )
		return;

	d_hiQualityFlag = hiQualityFlag;
	if( d_hiQualityFlag )
	{
		f00GLTexture::minFilterDefault( GL_LINEAR_MIPMAP_LINEAR );
		f00GLTexture::magFilterDefault( GL_LINEAR );
	}
	else
	{
		f00GLTexture::minFilterDefault( GL_NEAREST );
		f00GLTexture::magFilterDefault( GL_NEAREST );
	}

	for( int loop = (int)textureList.size() - 1; loop >= 0; loop-- )
		textureList[ loop ] -> loadRAMIntoGL();
}



