#ifndef f00ImageLoaderH
#define f00ImageLoaderH


#include <GL/gl.h>
#include <assert.h>


class f00ImageLoader
{
	public:

		f00ImageLoader()
		:	d_pImageRGBA( 0 )
		{
			resetRGBA();
		}


		virtual
		~f00ImageLoader()
		{
			resetRGBA();
		}


		// Must override!
		virtual bool
		loadImage( const char* pData, int lenData ) = 0;


		int
		getWidth()
		{
			return d_imageWidth;
		}


		int
		getHeight()
		{
			return d_imageHeight;
		}


		GLubyte*
		getRGBA( int& width, int& height )
		{
			if( d_pImageRGBA == 0 )
				return 0;
			width = d_imageWidth;
			height = d_imageHeight;
			return d_pImageRGBA;
		}


		GLubyte*
		extractRGBA( int& width, int& height )
		{
			GLubyte* pReturn = getRGBA( width, height );
			d_pImageRGBA = 0;
			resetRGBA();
			return pReturn;
		}


		void
		resetRGBA()
		{
			delete d_pImageRGBA;
			d_pImageRGBA = 0;
			d_imageWidth = -1;
			d_imageHeight = -1;
		}


	protected:

		void
		setSize( int width, int height )
		{
			d_imageWidth = width;
			d_imageHeight = height;
		}


		void
		setRGBA( GLubyte* pNewRGBA )
		{
			assert( d_pImageRGBA == 0 );
			d_pImageRGBA = pNewRGBA;
		}


	private:

		int						d_imageWidth;
		int						d_imageHeight;

		GLubyte*					d_pImageRGBA;


};


#endif

