#ifndef f00ObjectH
#define f00ObjectH


#include <GL/gl.h>

#include "f00Point3D.h"


//
// Base class for renderable/positional objects.
//

class f00Object
{
	// Stuff.
	public:
		f00Object(	GLfloat x = 0.0,
						GLfloat y = 0.0,
						GLfloat z = 0.0,
						GLfloat yaw = 0.0,
						GLfloat pitch = 0.0,
						GLfloat roll = 0.0	);

		virtual
		~f00Object();


		void
		render();

		void
		resetRot();

		void
		resetRotDelta();

		void
		resetPos();

		void
		resetPosDelta();


	// Accessors.
	public:
		GLfloat
		x()	{ return d_x; }

		GLfloat
		y()	{ return d_y; }

		GLfloat
		z()	{ return d_z; }


		void
		moveTo( GLfloat x, GLfloat y, GLfloat z )		{ d_x = x; d_y = y; d_z = z; }

		void
		moveBy( GLfloat x, GLfloat y, GLfloat z )		{ d_x += x; d_y += y; d_z += z; }


		GLfloat
		xDelta()	{ return d_xDelta; }

		GLfloat
		yDelta()	{ return d_yDelta; }

		GLfloat
		zDelta()	{ return d_zDelta; }


		void
		xDelta( GLfloat xDelta )		{ d_xDelta = xDelta; }

		void
		yDelta( GLfloat yDelta )		{ d_yDelta = yDelta; }

		void
		zDelta( GLfloat zDelta )		{ d_zDelta = zDelta; }



		GLfloat
		yaw()		{ return d_yaw; }

		GLfloat
		pitch()	{ return d_pitch; }

		GLfloat
		roll()	{ return d_roll; }


		void
		yawTo( GLfloat angle )		{ d_yaw = angle; }

		void
		yawBy( GLfloat angle )		{ d_yaw += angle; }


		void
		pitchTo( GLfloat angle )	{ d_pitch = angle; }

		void
		pitchBy( GLfloat angle )	{ d_pitch += angle; }


		void
		rollTo( GLfloat angle )		{ d_roll = angle; }

		void
		rollBy( GLfloat angle )		{ d_roll += angle; }


		GLfloat
		yawDelta()		{ return d_yawDelta; }

		GLfloat
		pitchDelta()	{ return d_pitchDelta; }

		GLfloat
		rollDelta()		{ return d_rollDelta; }


		void
		setDeltas( GLfloat yawDelta, GLfloat pitchDelta, GLfloat rollDelta )
		{
			d_yawDelta = yawDelta;
			d_pitchDelta = pitchDelta;
			d_rollDelta = rollDelta;
		}


	// Virtuals.
	public:
		virtual void
		renderObject();

		virtual void
		reset();

		virtual void
		update();


	// Data members.
	private:
		GLfloat d_yaw;
		GLfloat d_roll;
		GLfloat d_pitch;

		GLfloat d_yawDelta;
		GLfloat d_pitchDelta;
		GLfloat d_rollDelta;

		GLfloat d_x;
		GLfloat d_y;
		GLfloat d_z;

		GLfloat d_xDelta;
		GLfloat d_yDelta;
		GLfloat d_zDelta;

};


#endif

