#include <windows.h>
#pragma hdrstop


#include "f00RGBA.h"

#include <assert.h>
#include <string>
#include <stdio.h>//jun


//
// f00RGBA_f
//

f00RGBA_f::f00RGBA_f( GLfloat* pArray )
:	d_r( d_array[ 0 ] ),
	d_g( d_array[ 1 ] ),
	d_b( d_array[ 2 ] ),
	d_a( d_array[ 3 ] )
{
	assert( pArray );
	memcpy( d_array, pArray, 4 * sizeof( GLfloat ) );
}


f00RGBA_f::f00RGBA_f( GLfloat r, GLfloat g, GLfloat b, GLfloat a )
:	d_r( d_array[ 0 ] ),
	d_g( d_array[ 1 ] ),
	d_b( d_array[ 2 ] ),
	d_a( d_array[ 3 ] )
{
	setAll( r, g, b, a );
}


f00RGBA_f::f00RGBA_f( const f00RGBA_f& copyFrom )
:	d_r( d_array[ 0 ] ),
	d_g( d_array[ 1 ] ),
	d_b( d_array[ 2 ] ),
	d_a( d_array[ 3 ] )
{
	*this = copyFrom;
}


f00RGBA_f::f00RGBA_f( const std::string& extractFrom, const char * pFormat )
:	d_r( d_array[ 0 ] ),
	d_g( d_array[ 1 ] ),
	d_b( d_array[ 2 ] ),
	d_a( d_array[ 3 ] )
{
	set( extractFrom, pFormat );
}


f00RGBA_f&
f00RGBA_f::operator = ( const f00RGBA_f& copyFrom )
{
	memcpy( d_array, copyFrom.d_array, 4 * sizeof( GLfloat ) );
	return *this;
}


f00RGBA_f&
f00RGBA_f::set( const std::string& extractFrom, const char * pFormat )
{
	// FIXME  Ignore pFormat for now....

	// Rip it apart on ',' chars.
	const char * pString( extractFrom.c_str() );
	const char * pCommaPos1( strchr( pString, ',' ) );
	assert( pCommaPos1 );
	const char * pCommaPos2( strchr( pCommaPos1 + 1, ',' ) );
	assert( pCommaPos2 );
	const char * pCommaPos3( strchr( pCommaPos2 + 1, ',' ) );
	assert( pCommaPos3 );
	d_r = atof( pString ) / 255.0;
	d_g = atof( pCommaPos1 + 1 ) / 255.0;
	d_b = atof( pCommaPos2 + 1 ) / 255.0;
	d_a = atof( pCommaPos3 + 1 ) / 255.0;
	return *this;
}


std::string
f00RGBA_f::asString( const char * pFormat )
{
	// FIXME  Ignore pFormat for now....

	static char smallBuffer[ 100 ];
	sprintf( smallBuffer, "%f,%f,%f,%f", d_r * 255.0, d_g * 255.0, d_b * 255.0, d_a * 255.0 );
	return smallBuffer;
}




//
// f00RGBA_ub
//

f00RGBA_ub::f00RGBA_ub( GLubyte* pArray, bool makeCopy )
{
	assert( pArray );

	d_ownArray = makeCopy;
	if( d_ownArray )
	{
		d_pArray = new GLubyte[ 4 ];
		memcpy( d_pArray, pArray, 4 );
	}
	else
		d_pArray = pArray;

	d_pR = d_pArray + 0;
	d_pG = d_pArray + 1;
	d_pB = d_pArray + 2;
	d_pA = d_pArray + 3;
}


f00RGBA_ub::f00RGBA_ub( GLubyte r, GLubyte g, GLubyte b, GLubyte a )
{
	d_ownArray = true;
	d_pArray = new GLubyte[ 4 ];
	d_pR = d_pArray + 0;
	d_pG = d_pArray + 1;
	d_pB = d_pArray + 2;
	d_pA = d_pArray + 3;
	setAll( r, g, b, a );
}


f00RGBA_ub::f00RGBA_ub( const f00RGBA_ub& copyFrom )
{
	d_ownArray = false;
	d_pArray = 0;			// Not REALLY needed, but WTF.
	*this = copyFrom;
}


f00RGBA_ub::f00RGBA_ub( const std::string& extractFrom, const char * pFormat )
{
	d_ownArray = true;
	d_pArray = new GLubyte[ 4 ];
	d_pR = d_pArray + 0;
	d_pG = d_pArray + 1;
	d_pB = d_pArray + 2;
	d_pA = d_pArray + 3;
	set( extractFrom, pFormat );
}


f00RGBA_ub::~f00RGBA_ub()
{
	d_pR = 0;
	d_pG = 0;
	d_pB = 0;
	d_pA = 0;
	if( d_ownArray )
	{
		d_ownArray = false;
		delete[] d_pArray;
		d_pArray = 0;
	}
}


f00RGBA_ub&
f00RGBA_ub::operator = ( const f00RGBA_ub& copyFrom )
{
	// For now, just make sure they are the same types.
	assert( d_ownArray == copyFrom.d_ownArray );

	if( d_ownArray )
		memcpy( d_pArray, copyFrom.d_pArray, 4 );
	else
	{
		d_pArray = copyFrom.d_pArray;
		d_pR = copyFrom.d_pR;
		d_pG = copyFrom.d_pG;
		d_pB = copyFrom.d_pB;
		d_pA = copyFrom.d_pA;
	}

	return *this;
}


f00RGBA_ub&
f00RGBA_ub::set( const std::string& extractFrom, const char * pFormat )
{
	// FIXME  Ignore pFormat for now....

	// Rip it apart on ',' chars.
	const char * pString( extractFrom.c_str() );
	const char * pCommaPos1( strchr( pString, ',' ) );
	assert( pCommaPos1 );
	const char * pCommaPos2( strchr( pCommaPos1 + 1, ',' ) );
	assert( pCommaPos2 );
	const char * pCommaPos3( strchr( pCommaPos2 + 1, ',' ) );
	assert( pCommaPos3 );
	*d_pR = (GLubyte)atoi( pString );
	*d_pG = (GLubyte)atoi( pCommaPos1 + 1 );
	*d_pB = (GLubyte)atoi( pCommaPos2 + 1 );
	*d_pA = (GLubyte)atoi( pCommaPos3 + 1 );
	return *this;
}


std::string
f00RGBA_ub::asString( const char * pFormat )
{
	// FIXME  Ignore pFormat for now....

	static char smallBuffer[ 100 ];
	sprintf( smallBuffer, "%d,%d,%d,%d", int( *d_pR ), int( *d_pG ), int( *d_pB ), int( *d_pA ) );
	return smallBuffer;
}



