#ifndef f00RGBAH
#define f00RGBAH


#include <GL/gl.h>
#include <string>


class f00RGBA_f
{
	public:

		// Must point to an array of 4 or more GLfloats!  Ownership stays outside.
		f00RGBA_f( GLfloat* pArray );

		f00RGBA_f( GLfloat r = 0.0, GLfloat g = 0.0, GLfloat b = 0.0, GLfloat a = 0.0 );

		f00RGBA_f( const f00RGBA_f& copyFrom );

		f00RGBA_f( const std::string& extractFrom, const char * pFormat = 0 );


		f00RGBA_f&
		operator = ( const f00RGBA_f& copyFrom );


		// Ownership *stays* in f00RGBA_f!
		GLfloat *
		getArray()				{ return d_array; }


		GLfloat
		getR() const			{ return d_r; }

		GLfloat
		getG() const			{ return d_g; }

		GLfloat
		getB() const			{ return d_b; }

		GLfloat
		getA() const			{ return d_a; }


		GLfloat&
		getRRef()				{ return d_r; }

		GLfloat&
		getGRef()				{ return d_g; }

		GLfloat&
		getBRef()				{ return d_b; }

		GLfloat&
		getARef()				{ return d_a; }


		void
		setR( GLfloat r )		{ d_r = r; }

		void
		setG( GLfloat g )		{ d_g = g; }

		void
		setB( GLfloat b )		{ d_b = b; }

		void
		setA( GLfloat a )		{ d_a = a; }


		f00RGBA_f&
		setAll( GLfloat r, GLfloat g, GLfloat b, GLfloat a )
		{
			d_r = r;
			d_g = g;
			d_b = b;
			d_a = a;
			return *this;
		}


		// If pFormat == 0, default to "r,g,b,a".
		f00RGBA_f&
		set( const std::string& extractFrom, const char * pFormat = 0 );

		std::string
		asString( const char * pFormat = 0 );


	private:

		GLfloat			d_array[ 4 ];

		GLfloat&			d_r;
		GLfloat&			d_g;
		GLfloat&			d_b;
		GLfloat&			d_a;


};



class f00RGBA_ub
{
	public:

		// Must point to an array of 4 or more GLubytes!  Ownership stays outside.
		// If makeCopy is set false, a pointer to the array is cloned.
		f00RGBA_ub( GLubyte* pArray, bool makeCopy = true );

		f00RGBA_ub( GLubyte r = 0.0, GLubyte g = 0.0, GLubyte b = 0.0, GLubyte a = 0.0 );

		f00RGBA_ub( const f00RGBA_ub& copyFrom );

		f00RGBA_ub( const std::string& extractFrom, const char * pFormat = 0 );

		~f00RGBA_ub();


		f00RGBA_ub&
		operator = ( const f00RGBA_ub& copyFrom );


		// Ownership *stays* in f00RGBA_ub!
		GLubyte *
		getArray()				{ return d_pArray; }


		GLubyte
		getR() const			{ return *d_pR; }

		GLubyte
		getG() const			{ return *d_pG; }

		GLubyte
		getB() const			{ return *d_pB; }

		GLubyte
		getA() const			{ return *d_pA; }


		GLubyte&
		getRRef()				{ return *d_pR; }

		GLubyte&
		getGRef()				{ return *d_pG; }

		GLubyte&
		getBRef()				{ return *d_pB; }

		GLubyte&
		getARef()				{ return *d_pA; }


		void
		setR( GLubyte r )		{ *d_pR = r; }

		void
		setG( GLubyte g )		{ *d_pG = g; }

		void
		setB( GLubyte b )		{ *d_pB = b; }

		void
		setA( GLubyte a )		{ *d_pA = a; }


		f00RGBA_ub&
		setAll( GLubyte r, GLubyte g, GLubyte b, GLubyte a )
		{
			*d_pR = r;
			*d_pG = g;
			*d_pB = b;
			*d_pA = a;
			return *this;
		}


		// If pFormat == 0, default to "r,g,b,a".
		f00RGBA_ub&
		set( const std::string& extractFrom, const char * pFormat = 0 );

		std::string
		asString( const char * pFormat = 0 );


	private:

		GLubyte*			d_pArray;
		bool				d_ownArray;

		GLubyte*			d_pR;
		GLubyte*			d_pG;
		GLubyte*			d_pB;
		GLubyte*			d_pA;


};


#endif

