/*
#include <windows.h>
#pragma hdrstop
jun
*/

#include "f00Scene.h"

#include <GL/glut.h>
#include <assert.h>


#include "f00Cube.h"
#include "f00Light.h"

extern int window_ID;//jun

// Store the cube object here.
f00Cube* pCube = 0;

// Store some lights here.
f00Light* pSideLight = 0;
f00Light* pRearLight = 0;


f00Scene::f00Scene()
{
	d_enableAlphaBlend = true;
}


f00Scene::~f00Scene()
{
	//
	// Cleanup.
	//

	delete pCube;
	pCube = 0;

	delete( pSideLight );
	pSideLight = 0;

	delete( pRearLight );
	pRearLight = 0;
}


void
f00Scene::init()
{
	glClearColor( 0.3, 0.3, 0.3, 0.0 );
	glEnable( GL_DEPTH_TEST );

	glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

	resetCamera();

	glShadeModel( GL_SMOOTH );
	glEnable( GL_LIGHTING );
	glEnable( GL_CULL_FACE );

	glEnable( GL_COLOR_MATERIAL );
	glColor3f( 1.0, 1.0, 1.0 );

	// Off to the side a bit, behind the camera.
	pSideLight = new f00Light(	f00RGBA_f( 0.1, 0.1, 0.1 ),
										f00RGBA_f( 0.5, 0.5, 0.5 ),
										f00RGBA_f( 1.0, 1.0, 1.0 ),
										f00Point3D( -20.0, 10.0, 100.0 )		);
	glEnable( GL_LIGHT0 );

	// Behind everything.
	pRearLight = new f00Light(	f00RGBA_f( 0.05, 0.05, 0.05 ),
										f00RGBA_f( 0.3, 0.3, 0.3 ),
										f00RGBA_f( 0.5, 0.5, 0.5 ),
										f00Point3D( 100.0, -100.0, -1000.0 )	);
	glEnable( GL_LIGHT1 );

	// Make a cube, it will load it's own textures in....
	pCube = new f00Cube;

	// Spin it a bit.
	pCube -> setDeltas( 4.0, 1.0, 0.0 );
}


void
f00Scene::renderScene()
{
	// Tabula rasa.
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// Reset world.
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	// Enable transparency?
	if( d_enableAlphaBlend )
	{
		//
		// We cheat a bit here.  We know we will be using a cube, so we can force
		// the back-to-front order at render time.  (Required for this to work.)
		//
		glDisable( GL_CULL_FACE );
//		glBlendFunc( GL_SRC_ALPHA, GL_ONE );
		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		glDepthMask( GL_FALSE );
		glEnable( GL_BLEND );
	}
	else
	{
		glEnable( GL_CULL_FACE );
		glDepthMask( GL_TRUE );
		glDisable( GL_BLEND );
	}

	// Camera!
	d_camera.applyCamera();

	// Lights!
	pSideLight -> applyLight( 0 );
	pRearLight -> applyLight( 1 );

	// Action!
	pCube -> render();
}


//
// GLUT Interface
//

void
f00Scene::mouseFunc( int button, int state, int x, int y )
{
	if( button == GLUT_LEFT_BUTTON )
	{
		if( state == GLUT_DOWN )
		{
			// Do something on "Begin Drag/Click".
		}

		else if( state == GLUT_UP )
		{
			// "End Drag/Click".
		}
	}
}


void
f00Scene::resizeFunc( int w, int h )
{
	glViewport( 0, 0, w, h );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();

	// 50 degree FOV, aspect correction, near clip at 2.0, far clip at 1000.0.
	gluPerspective( 50, (double)w / max( (double)h, 1.0 ), 2.0, 1000.0 );

	glMatrixMode( GL_MODELVIEW );
}


void
f00Scene::keyFunc( unsigned char key )
{
	if( key >= '1' && key <= '9' )
	{
		GLfloat yawDelta( pCube -> yawDelta() );
		GLfloat pitchDelta( pCube -> pitchDelta() );
		GLfloat rollDelta( pCube -> rollDelta() );

		switch( key )
		{
			case '1':	yawDelta -= 1.0;		break;
			case '2':	yawDelta  = 0.0;		break;
			case '3':	yawDelta += 1.0;		break;

			case '4':	pitchDelta -= 1.0;	break;
			case '5':	pitchDelta  = 0.0;	break;
			case '6':	pitchDelta += 1.0;	break;

			case '7':	rollDelta -= 1.0;	break;
			case '8':	rollDelta  = 0.0;	break;
			case '9':	rollDelta += 1.0;	break;

			default:		assert( false );		break;		// WTFH?!  :-)
		}

		pCube -> setDeltas( yawDelta, pitchDelta, rollDelta );
	}
	else
	{
		switch( key )
		{
			case '0':
				pCube -> reset();
				break;


			case ' ':
				if( pCube -> isTexturedMode() )
					pCube -> setSprayColorMode();
				else
					pCube -> setTexturedMode();
				break;

			case 'a':
			case 'A':
				d_enableAlphaBlend = !d_enableAlphaBlend;
				break;

			case 'q':
			case 'Q':
				pCube -> setTextureQualityFlag( !pCube -> getTextureQualityFlag() );
				break;


			case 's':
			case 'S':
				resetCamera();
				break;

			case '[':
				pCube -> size( max( pCube -> size() - 0.5, 0.5 ) );
				break;

			case ']':
				pCube -> size( min( pCube -> size() + 0.5, 9.0 ) );
				break;

			case 27:				// ESC key.
				//jun start
				glutMouseFunc( NULL );
				glutReshapeFunc( NULL );
				glutKeyboardFunc( NULL );
				glutIdleFunc( NULL );
				window_ID = 0;
				//jun end
				exit( 0 );
				break;

		}
	}
}


void
f00Scene::idleFunc()
{
	pCube -> update();

	glutPostRedisplay();
}


void
f00Scene::resetCamera()
{
	d_camera.reset();

	// Move 20 units "out of the screen".
	d_camera.moveBy( 0.0, 0.0, 20.0 );
}



